/* Include the controller definition */

#include "group_size.h"
/* Function definitions for XML parsing */
#include <argos3/core/utility/configuration/argos_configuration.h>


/****************************************/
/****************************************/

CGroupSize::CGroupSize() :
    m_pcWheelAct(NULL),
    m_pcProxSens(NULL),
    m_pcGroundSensor(NULL),
    m_pcRNG(NULL)
   {}

/****************************************/
/****************************************/

void CGroupSize::Init(TConfigurationNode& t_node) {
    m_pcWheelAct = GetActuator<CCI_EPuckWheelsActuator>("epuck_wheels");
    m_pcProxSens = GetSensor<CCI_EPuckProximitySensor>("epuck_proximity");
    m_pcGroundSensor = GetSensor<CCI_EPuckGroundSensor>("epuck_ground");

    m_unNbCs = 0;
    m_doRandomWalk = false;
    m_fWheelVelocity = 5.0f;
    m_pcRNG = CRandom::CreateRNG("argos");
    m_bOAGoRight = false;

    try {
        TConfigurationNode randomWalkNode = GetNode(t_node, "random_walk");
        GetNodeAttribute(randomWalkNode, "alpha", m_cAlpha);
        m_cGoStraightAngleRange.Set(-ToRadians(m_cAlpha), ToRadians(m_cAlpha));
        GetNodeAttribute(randomWalkNode, "delta", m_fDelta);
        m_cGoStraightAngleRange.Set(-ToRadians(m_cAlpha), ToRadians(m_cAlpha));
        GetNodeAttribute(randomWalkNode, "velocity", m_fWheelVelocity);

    }
    catch(CARGoSException& ex) {
        THROW_ARGOSEXCEPTION_NESTED("Error initializing controller parameters.", ex);
    }

    m_unUnderRoofRotateCpt = 0;
    m_unStartGostraightCpt = 0;
    m_unUnderRoofRotateCpt = 0;
    m_unState = START;
}


/****************************************/
/****************************************/

void CGroupSize::ControlStep() {
   
    switch (m_unState) {
        case START: {
        				// Used in real expeirment (May 2015)
                        if (m_unStartGostraightCpt <= 40) {
                        //if (m_unStartGostraightCpt <= 1) {
                           m_pcWheelAct->SetLinearVelocity(10.0, 10.0);
                           m_unStartGostraightCpt++;
                        }
                        else {
                            m_unState = IN_WHITE_ZONE;
                        }
                        break;
            
                    }
        case IN_WHITE_ZONE: {
                                if (ObstacleDetected()) {
                                    m_bObstacleAvoidanceState = true;
                                    ObstacleAvoidance();
                                    return;
                                }

                                if (m_bObstacleAvoidanceState && !ObstacleDetected()) {
                                    m_bObstacleAvoidanceState = false;
                                }
                                if (!m_bObstacleAvoidanceState) {
                                    m_pcWheelAct->SetLinearVelocity(10.0, 10.0);
                                }

                                if (IsUnderRoof()) {
                                    m_unState = UNDER_ROOF;
                                }
                                break;
                        
                            }
        case UNDER_ROOF: {
                             if (m_unUnderRoofRotateCpt <= 15) {
                                 m_pcWheelAct->SetLinearVelocity(5.0, -5.0);
                                 ++m_unUnderRoofRotateCpt;
                             }
                             else {
                                 if (m_unUnderRoofGoStraightCpt <= 20) {
                                     m_pcWheelAct->SetLinearVelocity(10.0, 10.0);
                                     m_unUnderRoofGoStraightCpt++;
                                 }
                                 else {
                                    m_unUnderRoofRotateCpt = 0;
                                    m_unUnderRoofGoStraightCpt = 0;
                                    m_unState = IN_WHITE_ZONE;
                                 }
                             }
                            break;
                         }
        default: {
                     break;
                 }
    }
   // }
    ++m_unNbCs;

}

/****************************************/
/****************************************/
void CGroupSize::FollowVector(CVector2 sum, Real f_speed) {
      
    
    CRadians norm = sum.Angle().UnsignedNormalize();
    CRange<CRadians> left1(CRadians::PI, CRadians::TWO_PI);
    CRange<CRadians> right1(CRadians::ZERO, CRadians::PI);
    Real right = 0;
    Real left = 0;

    if (sum.GetX() != 0 || sum.GetY() != 0) {
        if (right1.WithinMinBoundExcludedMaxBoundExcluded(norm)) {
            right = 1;
        } 
        else {
            right = Max<Real > (-1.0f, Cos(norm));
        }
        if (left1.WithinMinBoundExcludedMaxBoundExcluded(norm)) {
            left = 1;
        } 
        else {
            left = Max<Real > (-1.0f, Cos(norm));
        }
    }
    if (std::abs(left) + std::abs(right) == 0.0f) {
        m_pcWheelAct->SetLinearVelocity(0, 0);
    } 
    else {
        Real p = f_speed / std::max(std::abs(left), std::abs(right));
        Real l = p * left;
        Real r = p * right;
        m_pcWheelAct->SetLinearVelocity(l, r);
    }
}


/****************************************/
/****************************************/
bool CGroupSize::IsUnderRoof() {
    CCI_EPuckGroundSensor::SReadings readings = m_pcGroundSensor->GetReadings();
    return readings[0] <= 0.1 || readings[1] <= 0.1 || readings[2] <= 0.1; 
    /*  ///Real fLightThreshold = 0.00025;
    Real fLightThreshold = 0.0012;
    const CCI_EPuckLightSensor::TReadings& tReadings = m_pcLightSens->GetReadings();
    Real fAccumValue = 0.0;
    fAccumValue += tReadings[0].Value;
    fAccumValue += tReadings[7].Value;
    fAccumValue += tReadings[1].Value;
    fAccumValue += tReadings[6].Value;
    fAccumValue += tReadings[5].Value;
    fAccumValue += tReadings[2].Value;



    LOG << "Light value: " << fAccumValue << std::endl;

    if (fAccumValue < fLightThreshold) {
        LOG << "Should go out" << std::endl;
    }
    return fAccumValue < fLightThreshold;
*/
}
/****************************************/
/****************************************/
void CGroupSize::GoOutside() {

    if (m_unStartGostraightCpt <= 20) {
        ++m_unStartGostraightCpt;
    }
    else {
        if (m_unUnderRoofRotateCpt <= 5) {
            m_pcWheelAct->SetLinearVelocity(-5.0,5.0);
            LOGERR << "Rotating" << std::endl;
            ++m_unUnderRoofRotateCpt;
            m_unStartGostraightCpt = 0;
        }
        else {
;
            m_unUnderRoofRotateCpt = 0;
        }
    }

  //  FollowVector(10*vLight, 10.0);
}

/****************************************/
/****************************************/
void CGroupSize::ObstacleAvoidance() {
    Real velocity = 3.0f;

	 if (m_bOAGoRight) {
	       m_pcWheelAct->SetLinearVelocity(velocity, -velocity);
	    } else {
	    	m_pcWheelAct->SetLinearVelocity(-velocity, velocity);
	    }
}

/****************************************/
/****************************************/

bool CGroupSize::ObstacleDetected() {
	
	Real fProximityThreshold = .001;
	// Used in real experiment (experiments done in may 2015))
	//Real fProximityThreshold = .1;

	const CCI_EPuckProximitySensor::TReadings& tReadings = m_pcProxSens->GetReadings();

	std::cout << tReadings[0].Value << " " << tReadings[1].Value  << " " << tReadings[6].Value << " " << tReadings[7].Value << std::endl;

	if (tReadings[0].Value > fProximityThreshold && tReadings[7].Value > fProximityThreshold) {
		if (m_pcRNG->Bernoulli(0.3)) {
			m_bOAGoRight = true;
		}
		else {
			m_bOAGoRight = false;
		}

	}

	else if (tReadings[0].Value > fProximityThreshold && tReadings[1].Value > fProximityThreshold) {
		m_bOAGoRight = true;
	}
	else if (tReadings[7].Value > fProximityThreshold && tReadings[6].Value > fProximityThreshold) {
		m_bOAGoRight = false;
	}

	else if (tReadings[0].Value > fProximityThreshold && tReadings[1].Value <= fProximityThreshold && tReadings[6].Value <= fProximityThreshold
			&& tReadings[7].Value <= fProximityThreshold) {
		m_bOAGoRight =true;
		return true;
	}

	else if (tReadings[7].Value > fProximityThreshold && tReadings[1].Value <= fProximityThreshold && tReadings[6].Value <= fProximityThreshold
			&& tReadings[0].Value <= fProximityThreshold) {
		m_bOAGoRight =false;
		return true;
	}
	else if (tReadings[1].Value > fProximityThreshold && tReadings[0].Value <= fProximityThreshold && tReadings[6].Value <= fProximityThreshold
			&& tReadings[7].Value <= fProximityThreshold) {
		m_bOAGoRight =true;
		return true;
	}
	else if (tReadings[6].Value > fProximityThreshold && tReadings[1].Value <= fProximityThreshold && tReadings[0].Value <= fProximityThreshold
			&& tReadings[7].Value <= fProximityThreshold) {
		m_bOAGoRight =false;
		return true;
	}


	else {
		return false;
	}

	return true;


}


/****************************************/
/****************************************/

REGISTER_CONTROLLER(CGroupSize, "group_size")
