#ifndef EPUCK_PSYCHO_PASSIVE_H
#define EPUCK_PSYCHO_PASSIEV_H

#include <argos3/core/control_interface/ci_controller.h>

#include <argos3/plugins/robots/e-puck/control_interface/ci_epuck_wheels_actuator.h>
#include <argos3/plugins/robots/e-puck/control_interface/ci_epuck_proximity_sensor.h>
#include <argos3/core/utility/math/vector2.h>
#include <argos3/core/utility/math/rng.h>
#include <argos3/plugins/robots/generic/control_interface/ci_ground_sensor.h>
#include <argos3/plugins/robots/e-puck/control_interface/ci_epuck_ground_sensor.h>

using namespace argos;

#define STATE_WALKING "walking"
#define STATE_ROTATING "rotating"

class CGroupSize  : public CCI_Controller {
   enum BehaviouralState {
        START,
        IN_WHITE_ZONE,
        UNDER_ROOF
    };


         public: /* argos related */
        CGroupSize();
        virtual ~CGroupSize() {};

        virtual void Init(TConfigurationNode& t_node);
        virtual void ControlStep();
        virtual void Reset() {};
        virtual void Destroy() {};

    protected: /* functions */

        void RandomWalk();
        bool ObstacleDetected();
        void ObstacleAvoidance();
        bool IsUnderRoof();
        void GoOutside();

        void FollowVector(CVector2 sum, Real f_speed);


    private: /*  members */
        CCI_EPuckWheelsActuator* m_pcWheelAct;
        CCI_EPuckProximitySensor* m_pcProxSens;
        CCI_EPuckGroundSensor* m_pcGroundSensor;

        UInt8 NB_GROUPS; 

        std::vector<std::vector<std::string> > m_vGroups;


        std::vector<UInt16> m_vStartGroup;
        UInt64 m_unNbCs;
        
        bool m_doRandomWalk;

        std::string m_sCurrentState;

        CRange<CRadians> m_cGoStraightAngleRange;
        Real m_fDelta;
        CDegrees m_cAlpha;
        Real m_fWheelVelocity;
        bool m_bOAGoRight;
        CRandom::CRNG *m_pcRNG;

        bool m_bObstacleAvoidanceState;

        UInt8 m_unStartGostraightCpt;
        UInt8 m_unUnderRoofRotateCpt;
        UInt8 m_unUnderRoofGoStraightCpt;
        
        UInt16 m_unStopExperiment;

        BehaviouralState m_unState;


};
#endif
